unit EzDateEdit;

{********************************************************************

    The TEzDateEdit component is Copyright 1996
    by Tim Borman, All Rights Reserved.

    Version 0.9 (beta)

    This component may be freely distributed. It may be used
    or abused in any way desired provided that the author's
    copyright is recognized and respected.
    The author will not be held responsible for any problems
    arising from the use or mis-use of this component.

    If you find any bugs, have any suggestions for improvements,
    or make improvements yourself please tell me about them.
    Tim Borman, 72633.722@Compuserve.Com .


This component is designed to make date entry easy. No more
requirements to enter dates as carefully crafted sets of numbers.
Just tell it the field order (MDY etc), which fields are required,
and how to format the result. Months can be entered as numbers
or as month names (needing only enough letters to identify the
month). Field separators can be anything except a letter or number.
Years default to the current century if only two digits are entered.
Some examples, all using the default properties:
"D 1 96" -> Dec 1, 1996
"1/1/2001" -> Jan 1, 2001
"8/13" -> Aug 13, 1996
"ju 1" -> Jun 1, 1996

Credit where credit is due department:
I don't know who wrote the original freeware "DateEdit" but that
served as the basis for this. Not much of the original code remains
but much of the functionality is the same (especially in the popup)
and he/she deserves thanks. 

Usage notes:
Don't assume that the Date property is valid if the component is
currently focused. The Date property is only set when 1) a date is
selected from the popup, 2) the componenent is exited, or 3) when
a value is set in Date or SetDateText().  You can force the Date
to be updated by calling SetDateText(Text) if needed.

Properties:
  AllowBlank
    Whether a blank entry is allowed.
    Default: True;
  DateOrder
    The order of the MDY fields.
    Default: Windows/CurrentUser default.
  RequiredFields
    Which of the MDY fields are required. Non-required fields
    will be filled in with the current system date if not entered.
    Default: MD
  DateFormat
    Display format of date. See FormatDateTime function in Delphi.
    If blank, uses Delphi's ShortDateFormat.
    Default: Blank.
  TodayString
    A string that can be entered to fill in the current date.
    Default:'/'
  Date
    Runtime only. The entered date, obviously, as TDateTime.
Procedures
  SetDateText
    The string is checked just as text entered into the edit
    would be.
  Popup
    Pops up the little calendar, same as clicking the button.
Events
  OnGetText
    Called immediately before a text value is processed. You can
    use this to alter the text before processing.
  OnSetDate
    Called after the date is set.

History:
  Oct   8, 96 - changed the TDateField type casts in DbEzDateEdit
                to TDateTimeField.
              - when linked to a TDateTimeField only the date portion
                will be changed by DbEzDateEdit.  
  Sept 30, 96 - added OnGetText event
              - 'Dec' was not being accepted as a valid month.
  Sept 23, 96 - first attempt to call it finished.


********************************************************************}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, StdCtrls, EzDatePopup, Buttons, Dialogs;

type

  TDateButton = class( TBitBtn )
  private
  protected
     procedure Click; override;
  public
  published
  end;

  TEzDateOrder = (fdo_MDY, fdo_DMY, fdo_YMD);
  TEzDateRequiredField = (fdr_Year, fdr_Month, fdr_Day);
  TEzDateRequiredFields = set of TEzDateRequiredField;
  TEzDateGetTextEvent = procedure(Sender:TObject ; var Text:String) of Object;

  TEzDateEdit = class( TEdit )
  private
    fAllowBlank:Boolean;
    fButton: TDateButton;
    fDate: TDateTime;
    fDateFormat:String;
    fDateOrder:TEzDateOrder;
    fRequiredFields:TEzDateRequiredFields;
    fTodayString:String;
    fOnSetDate:TNotifyEvent;
    fOnGetText:TEzDateGetTextEvent;
  protected
    procedure WMSize( var Message: TWMSize ); message WM_SIZE;
    procedure SetDate( Value: TDateTime );
    procedure DoExit; override;
  public
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;
    procedure CreateParams( var Params: TCreateParams ); override;
    property Date: TDateTime read fDate write SetDate;
    procedure SetDateText(Value:String); dynamic;
    procedure SetDateFormat(Value:String);
    procedure SetDateOrder(Value:TEzDateOrder);
    procedure Popup; dynamic;
  published
    property AllowBlank:Boolean read fAllowBlank write fAllowBlank default True;
    property DateFormat:String read fDateFormat write SetDateFormat;
    property DateOrder:TEzDateOrder read fDateOrder write SetDateOrder;
    property RequiredFields:TEzDateRequiredFields read fRequiredFields write fRequiredFields;
    property TodayString: String read fTodayString write fTodayString;
    property OnGetText:TEzDateGetTextEvent read fOnGetText write fOnGetText;
    property OnSetDate:TNotifyEvent read fOnSetDate write fOnSetDate;
  end;

procedure Register;

var
  frmCalendar: TfrmEzDatePopup;

implementation

{--- TDateButton ---}
procedure TDateButton.Click;
var
  editParent: TEzDateEdit;
begin
  editParent := TEzDateEdit( Parent );
  frmCalendar := TfrmEzDatePopup.Create( editParent );
  if frmCalendar.Execute(EditParent.fDate) = mrOK
  then EditParent.SetDate(EditParent.fDate);
  frmCalendar.Free;
  inherited Click;
  EditParent.SetFocus;
end;

{--- TEzDateEdit ---}

constructor TEzDateEdit.Create( AOwner: TComponent );
var
  c:array[0..1] of Char;
begin
  inherited Create( AOwner );
  FButton := TDateButton.Create( Self );
  FButton.Width := 17 * 3 div 4;
  FButton.Height := 17;
  FButton.Visible := TRUE;
  FButton.Parent := self;
  FButton.Caption := '...';
  fButton.TabStop := False;
  fAllowBlank := True;
  ControlStyle := ControlStyle - [csSetCaption];
  fDateFormat := '';
  C[0] := '0'; {default}
  GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_IDATE, c, 2);
  fDateOrder := TEzDateOrder(Ord(C[0])-Ord('0'));
  fRequiredFields := [fdr_Month, fdr_Day];
  fTodayString := '/';
end;

procedure TEzDateEdit.CreateParams( var Params: TCreateParams );
begin
  inherited CreateParams( Params );
  Params.Style := Params.Style or WS_CLIPCHILDREN;
end;

destructor TEzDateEdit.Destroy;
begin
  FButton.Free;
  inherited Destroy;
end;

procedure TEzDateEdit.WMSize( var Message: TWMSize );
var
  nWidth: integer;
begin
  FButton.Height := Height - 3;
  nWidth := Height - 3;
  if nWidth > Width * 0.15 then nWidth := Round(Width * 0.15);
  FButton.Width := nWidth;
  FButton.Left := Width - nWidth - 3;
  FButton.Refresh;
end;

procedure TEzDateEdit.SetDate( Value: TDateTime );
var
  S:String;
begin
  fDate := Value;
  Modified := TRUE;
  if fDate = 0
  then Text := ''
  else begin
    if fDateFormat = ''
    then S := ShortDateFormat
    else S := fDateFormat;
    Text := FormatDateTime( S, fDate );
    end; {else}
  if Assigned(fOnSetDate)
  then fOnSetDate(Self);
end;

procedure TEzDateEdit.SetDateFormat(Value:String);
begin
  fDateFormat := Value;
  SetDate(fDate);
end;

procedure TEzDateEdit.SetDateOrder(Value:TEzDateOrder);
begin
  fDateOrder := Value;
  SetDate(fDate);
end;

procedure TEzDateEdit.DoExit;
begin
  try
    SetDateText(Text);
  except
     SetFocus;
     raise;
  end;
  inherited DoExit;
end;

procedure TEzDateEdit.Popup;
begin
  fButton.Click;
end;

procedure TEzDateEdit.SetDateText(Value:String);
var
  StringList:TStringList;
  Field,I1,I2:Integer;
  S:String;
  Error:Boolean;
  ThisYear,ThisMonth,ThisDay:Word;
  Year,Month,Day:Word;
  FieldOrder:String[3];
begin
  if Assigned(fOnGetText) then fOnGetText(Self, Value);
  if Value = '' then begin
    if fAllowBlank then begin
      fDate := 0;
      Text := '';
      Exit;
      end {then}
    else raise Exception.Create('Blank Date not Allowed');
    end; {if}
  if Value = TodayString then begin
    fDate := SysUtils.Date;
    if fDateFormat = ''
    then S := ShortDateFormat
    else S := fDateFormat;
    Text := FormatDateTime( S, fDate );
    end {then}
  else begin
    DecodeDate(SysUtils.Date, ThisYear,ThisMonth,ThisDay);
    Value := UpperCase(Value);
    StringList := TStringList.Create;
    try
      I1 := 1; {first, parse the string}
      while (I1 <= Length(Value))
        and not (Value[I1] in ['0'..'9', 'A'..'Z'])
      do Inc(I1);
      while I1 <= Length(Value) do begin
        I2 := I1;
        while (I2 <= Length(Value))
          and (Value[I2] in ['0'..'9', 'A'..'Z'])
        do Inc(I2);
        StringList.Add(Copy(Value, I1, I2-I1));
        while (I2 <= Length(Value))
          and not (Value[I2] in ['0'..'9', 'A'..'Z'])
        do Inc(I2);
        I1 := I2;
        end; {while}
      case fDateOrder of
        fdo_MDY:FieldOrder := 'MDY';
        fdo_DMY:FieldOrder := 'DMY';
        fdo_YMD:FieldOrder := 'YMD';
        end; {case}
      for Field := 1 to Length(FieldOrder) do begin {now analyse it}
        if StringList.Count > 0
        then S := StringList[0]
        else S := '';
        Error := False;
        case FieldOrder[Field] of
          'M':begin
            if (S = '') or (S[1] in ['0'..'9']) then begin {numeric month}
              try
                if S = ''
                then Month := 0
                else Month := StrToInt(S);
              except
                Month := 0;
                Error := True
              end;
              if not (Month in [1..12]) then Month := 0;
              end {then}
            else begin {one or more letters in month}
              Month := 0;
              I1 := 1;
              S := Copy(S, 1, 3); {only 3 chars needed to ID month}
              Error := True;
              repeat
                if S = UpperCase(Copy(ShortMonthNames[I1], 1, Length(S))) then begin
                  Month := I1;
                  I1 := 13;
                  Error := False;
                  end {then}
                else Inc(I1);
              until I1 = 13;
              end; {else}
            if Month = 0 then begin
              if fdr_Month in fRequiredFields
              then Error := True
              else Month := ThisMonth;
              end {then}
            else if StringList.Count > 0 then StringList.Delete(0);
            if Error then begin
              if S = ''
              then raise Exception.Create('Invalid Date "'+Value+'"')
              else raise Exception.Create('Invalid Month "'+S+'"');
              end; {if}
            end;
          'Y':begin
            try
              if S = ''
              then Year := 0
              else Year := StrToInt(S);
            except
              Year := 0;
              Error := True
            end;
            {default to current century if anything entered}
            if (Year < 100) and (S <> '')
            then Year := Year + (ThisYear div 100 * 100);
            if Year = 0 then begin
              if fdr_Year in fRequiredFields
              then Error := True
              else Year := ThisYear;
              end {then}
            else if StringList.Count > 0 then StringList.Delete(0);
            if Error then begin
              if S = ''
              then raise Exception.Create('Invalid Date "'+Value+'"')
              else raise Exception.Create('Invalid Year "'+S+'"');
              end; {if}
            end;
          'D':begin
            try
              if S = ''
              then Day := 0
              else Day := StrToInt(S);
            except
              Day := 0;
              Error := True
            end;
            if not (Day in [1..31]) then Day := 0;
            if Day = 0 then begin
              if fdr_Day in fRequiredFields
              then Error := True
              else Day := ThisDay;
              end {then}
            else if StringList.Count > 0 then StringList.Delete(0);
            if Error then begin
              if S = ''
              then raise Exception.Create('Invalid Date "'+Value+'"')
              else raise Exception.Create('Invalid Day "'+S+'"');
              end; {if}
            end;
          end; {case}
        end; {for}
      try
        fDate := EncodeDate(Year, Month, Day);
        if fDateFormat = ''
        then S := ShortDateFormat
        else S := fDateFormat;
        Text := FormatDateTime( S, fDate );
      except
        raise Exception.Create('Invalid Date "'+Value+'"');
      end;
    finally
      StringList.Free;
      end;
    end; {else}
end;

{-------------------------}

procedure Register;
begin
  RegisterComponents('Additional', [TEzDateEdit]);
end;

end.
